@@ -1,16 +1,19 @@
-/* Copyright (C) 2001-2009 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
+
 /* This is the code that is used to convert the various PDF and PS CIE
    based color spaces to ICC profiles.  This enables the use of an
    external CMS that is ICC centric to be used for ALL color management.
@@ -118,6 +121,8 @@ Note: All profile data must be encoded as big-endian
 #include "string_.h"
 #include "gsmemory.h"
 #include "gx.h"
+#include <gp.h>
+
 #include "gxistate.h"
 #include "gstypes.h"
 #include "gscspace.h"
@@ -448,8 +453,8 @@ save_profile(unsigned char *buffer, char filename[], int buffer_size)
     char full_file_name[50];
     FILE *fid;
 
-    sprintf(full_file_name,"%d)Profile_%s.icc",icc_debug_index,filename);
-    fid = fopen(full_file_name,"wb");
+    gs_sprintf(full_file_name,"%d)Profile_%s.icc",icc_debug_index,filename);
+    fid = gp_fopen(full_file_name,"wb");
     fwrite(buffer,sizeof(unsigned char),buffer_size,fid);
     fclose(fid);
     icc_debug_index++;
@@ -1977,7 +1982,7 @@ gsicc_create_fromdefg(const gs_color_space *pcs, unsigned char **pp_buffer_in,
 
     /* Fill in the uncommon stuff */
     gsicc_create_init_luta2bpart(&icc_luta2bparts);
-    header->colorSpace = icSig4colorData;
+    header->colorSpace = icSigCmykData;
     icc_luta2bparts.num_in = 4;
 
     /* The a curves stored as def procs */
@@ -2039,7 +2044,7 @@ gsicc_create_fromdef(const gs_color_space *pcs, unsigned char **pp_buffer_in,
 
     gsicc_create_init_luta2bpart(&icc_luta2bparts);
 
-    header->colorSpace = icSig3colorData;
+    header->colorSpace = icSigRgbData;
     icc_luta2bparts.num_in = 3;
 
     /* The a curves stored as def procs */
