@@ -1,16 +1,19 @@
-/* Copyright (C) 2001-2011 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
+
 /* gsicc interface to littleCMS */
 
 #ifndef LCMS_USER_ALLOC
@@ -18,9 +21,11 @@
 #endif
 
 #include "gsicc_cms.h"
-#include "gserror.h"
 #include "lcms.h"
 #include "gslibctx.h"
+#include "gserrors.h"
+#include "gp.h"
+
 
 #define DUMP_CMS_BUFFER 0
 #define DEBUG_LCMS_MEM 0
@@ -68,13 +73,19 @@ gscms_get_numberclrtnames(gcmmhprofile_t profile)
 
 /* Get the nth colorant name in the clrt tag */
 char*
-gscms_get_clrtname(gcmmhprofile_t profile, int colorcount)
+gscms_get_clrtname(gcmmhprofile_t profile, int colorcount, gs_memory_t *memory)
 {
     LPcmsNAMEDCOLORLIST lcms_names;
+    int length;
+    char *name;
 
     lcms_names = cmsReadColorantTable(profile, icSigColorantTableTag);
     if (colorcount+1 > lcms_names->nColors) return(NULL);
-    return(lcms_names->List[colorcount].Name);
+    length = strlen(lcms_names->List[colorcount].Name);
+    name = (char*) gs_alloc_bytes(memory, length, "gscms_get_clrtname");
+    if (name)
+        strcpy(name, lcms_names->List[colorcount].Name);
+    return name;
 }
 
 /* Get the device space associated with this profile */
@@ -103,7 +114,7 @@ gscms_get_profile_data_space(gcmmhprofile_t profile)
 
 /* Get ICC Profile handle from buffer */
 gcmmhprofile_t
-gscms_get_profile_handle_mem(unsigned char *buffer, unsigned int input_size)
+gscms_get_profile_handle_mem(gs_memory_t *mem, unsigned char *buffer, unsigned int input_size)
 {
     cmsSetErrorHandler((cmsErrorHandlerFunction) gscms_error);
     return(cmsOpenProfileFromMem(buffer,input_size));
@@ -111,14 +122,14 @@ gscms_get_profile_handle_mem(unsigned char *buffer, unsigned int input_size)
 
 /* Get ICC Profile handle from file ptr */
 gcmmhprofile_t
-gscms_get_profile_handle_file(const char *filename)
+gscms_get_profile_handle_file(gs_memory_t *mem, const char *filename)
 {
     return(cmsOpenProfileFromFile(filename, "r"));
 }
 
 /* Transform an entire buffer */
 void
-gscms_transform_color_buffer(gsicc_link_t *icclink,
+gscms_transform_color_buffer(gx_device *dev, gsicc_link_t *icclink,
                              gsicc_bufferdesc_t *input_buff_desc,
                              gsicc_bufferdesc_t *output_buff_desc,
                              void *inputbuffer,
@@ -201,8 +212,8 @@ gscms_transform_color_buffer(gsicc_link_t *icclink,
         cmsDoTransform(hTransform,inputpos,outputpos,
                         input_buff_desc->plane_stride);
 #if DUMP_CMS_BUFFER
-        fid_in = fopen("CM_Input.raw","ab");
-        fid_out = fopen("CM_Output.raw","ab");
+        fid_in = gp_fopen("CM_Input.raw","ab");
+        fid_out = fp_fopen("CM_Output.raw","ab");
         fwrite((unsigned char*) inputbuffer,sizeof(unsigned char),
                                 input_buff_desc->plane_stride * 
                                 input_buff_desc->num_chan, fid_in);
@@ -221,8 +232,8 @@ gscms_transform_color_buffer(gsicc_link_t *icclink,
             outputpos += output_buff_desc->row_stride;
         }
 #if DUMP_CMS_BUFFER
-        fid_in = fopen("CM_Input.raw","ab");
-        fid_out = fopen("CM_Output.raw","ab");
+        fid_in = gp_fopen("CM_Input.raw","ab");
+        fid_out = gp_fopen("CM_Output.raw","ab");
         fwrite((unsigned char*) inputbuffer,sizeof(unsigned char),
                                 input_buff_desc->row_stride,fid_in);
         fwrite((unsigned char*) outputbuffer,sizeof(unsigned char),
@@ -237,11 +248,8 @@ gscms_transform_color_buffer(gsicc_link_t *icclink,
    of elements of size gx_device_color. It is up to the caller to make sure
    the proper allocations for the colors are there. */
 void
-gscms_transform_color(gsicc_link_t *icclink,
-                             void *inputcolor,
-                             void *outputcolor,
-                             int num_bytes,
-                             void **contextptr)
+gscms_transform_color(gx_device *dev, gsicc_link_t *icclink, void *inputcolor,
+                      void *outputcolor, int num_bytes)
 {
     cmsHTRANSFORM hTransform = (cmsHTRANSFORM) icclink->link_handle;
     DWORD dwInputFormat,dwOutputFormat,curr_input,curr_output;
@@ -264,17 +272,15 @@ gscms_transform_color(gsicc_link_t *icclink,
 /* Get the link from the CMS. TODO:  Add error checking */
 gcmmhlink_t
 gscms_get_link(gcmmhprofile_t  lcms_srchandle,
-                    gcmmhprofile_t lcms_deshandle,
-                    gsicc_rendering_param_t *rendering_params)
+               gcmmhprofile_t lcms_deshandle,
+               gsicc_rendering_param_t *rendering_params,
+               gs_memory_t *mem)
 {
     DWORD src_data_type,des_data_type;
     icColorSpaceSignature src_color_space,des_color_space;
     int src_nChannels,des_nChannels;
     int lcms_src_color_space, lcms_des_color_space;
 
-    /* Check for case of request for a transfrom from a device link profile
-       in that case, the destination profile is NULL */
-
    /* First handle all the source stuff */
     src_color_space  = cmsGetColorSpace(lcms_srchandle);
     lcms_src_color_space = _cmsLCMScolorSpace(src_color_space);
@@ -305,48 +311,76 @@ gscms_get_link(gcmmhprofile_t  lcms_srchandle,
     /* cmsFLAGS_HIGHRESPRECALC)  cmsFLAGS_NOTPRECALC  cmsFLAGS_LOWRESPRECALC*/
 }
 
-/* Get the link from the CMS, but include proofing.
-    We need to note that as an option in the rendering params.  If we are doing
-    transparency, that would only occur at the top of the stack
-TODO:  Add error checking */
+/* Get the link from the CMS, but include proofing and/or a device link  
+   profile. */
 gcmmhlink_t
-gscms_get_link_proof(gcmmhprofile_t  lcms_srchandle,
-                    gcmmhprofile_t lcms_deshandle, gcmmhprofile_t lcms_proofhandle,
-                    gsicc_rendering_param_t *rendering_params)
+gscms_get_link_proof_devlink(gcmmhprofile_t lcms_srchandle,
+                             gcmmhprofile_t lcms_proofhandle,
+                             gcmmhprofile_t lcms_deshandle, 
+                             gcmmhprofile_t lcms_devlinkhandle, 
+                             gsicc_rendering_param_t *rendering_params,
+                             bool src_dev_link,
+                             gs_memory_t *mem)
 {
     DWORD src_data_type,des_data_type;
     icColorSpaceSignature src_color_space,des_color_space;
     int src_nChannels,des_nChannels;
+    int lcms_src_color_space, lcms_des_color_space;
+    cmsHPROFILE hProfiles[5]; 
+    int nProfiles = 0;
 
-    /* Get the data types */
+   /* First handle all the source stuff */
     src_color_space  = cmsGetColorSpace(lcms_srchandle);
-    des_color_space  = cmsGetColorSpace(lcms_deshandle);
+    lcms_src_color_space = _cmsLCMScolorSpace(src_color_space);
+    /* littlecms returns -1 for types it does not (but should) understand */
+    if (lcms_src_color_space < 0) lcms_src_color_space = 0;
     src_nChannels = _cmsChannelsOf(src_color_space);
+    /* For now, just do single byte data, interleaved.  We can change this
+      when we use the transformation. */
+    src_data_type = (COLORSPACE_SH(lcms_src_color_space)|
+                        CHANNELS_SH(src_nChannels)|BYTES_SH(2));    
+    if (lcms_deshandle != NULL) {
+        des_color_space  = cmsGetColorSpace(lcms_deshandle);
+    } else {
+        /* We must have a device link profile. */
+        des_color_space = cmsGetPCS(lcms_deshandle);
+    }
+    lcms_des_color_space = _cmsLCMScolorSpace(des_color_space);
+    if (lcms_des_color_space < 0) lcms_des_color_space = 0;
     des_nChannels = _cmsChannelsOf(des_color_space);
-    /* For now, just do single byte data, interleaved.  We can change this when we
-       use the transformation. */
-    src_data_type= (CHANNELS_SH(src_nChannels)|BYTES_SH(1));
-    des_data_type= (CHANNELS_SH(des_nChannels)|BYTES_SH(1));
-    /* Create the link.  Note the gamut check alarm */
-    return(cmsCreateProofingTransform(lcms_srchandle, src_data_type,
-                                      lcms_deshandle, des_data_type,
-                                      lcms_proofhandle,
-                                      rendering_params->rendering_intent,
-                                      INTENT_ABSOLUTE_COLORIMETRIC,
-                                      cmsFLAGS_GAMUTCHECK | cmsFLAGS_SOFTPROOFING ));
+    des_data_type = (COLORSPACE_SH(lcms_des_color_space)|
+                        CHANNELS_SH(des_nChannels)|BYTES_SH(2));
+    /* lcms proofing transform has a clunky API and can't include the device 
+       link profile if we have both. So use cmsCreateMultiprofileTransform 
+       instead and round trip the proofing profile. */
+    hProfiles[nProfiles++] = lcms_srchandle;
+    if (lcms_proofhandle != NULL) {
+        hProfiles[nProfiles++] = lcms_proofhandle;
+        hProfiles[nProfiles++] = lcms_proofhandle;
+    }
+    hProfiles[nProfiles++] = lcms_deshandle;
+    if (lcms_devlinkhandle != NULL) {
+        hProfiles[nProfiles++] = lcms_devlinkhandle;
+    }
+    return(cmsCreateMultiprofileTransform(hProfiles, nProfiles, src_data_type, 
+                                          des_data_type, rendering_params->rendering_intent, 
+                                          (cmsFLAGS_BLACKPOINTCOMPENSATION | 
+                                           cmsFLAGS_HIGHRESPRECALC |
+                                           cmsFLAGS_NOTCACHE)));
 }
 
 /* Do any initialization if needed to the CMS */
-void
-gscms_create(void **contextptr)
+int
+gscms_create(gs_memory_t *memory)
 {
     /* Set our own error handling function */
     cmsSetErrorHandler((cmsErrorHandlerFunction) gscms_error);
+    return 0;
 }
 
 /* Do any clean up when done with the CMS if needed */
 void
-gscms_destroy(void **contextptr)
+gscms_destroy(gs_memory_t *memory)
 {
     /* Nothing to do here for lcms */
 }
