@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Color image rendering */
 
 #include "gx.h"
@@ -64,7 +66,11 @@ gs_image_class_4_color(gx_image_enum * penum)
 {
     bool std_cmap_procs;
     int code;
+#if USE_FAST_HT_CODE
+    bool use_fast_thresh = true;
+#else
     bool use_fast_thresh = false;
+#endif
     bool is_planar_dev = false;
 
     if (penum->use_mask_color) {
@@ -117,7 +123,7 @@ gs_image_class_4_color(gx_image_enum * penum)
         cmm_dev_profile_t *dev_profile;
 
         code = dev_proc(penum->dev, get_profile)(penum->dev, &dev_profile);
-        des_num_comp = dev_profile->device_profile[0]->num_comps;
+        des_num_comp = gsicc_get_device_profile_comps(dev_profile);
         penum->icc_setup.need_decode = false;
         /* Check if we need to do any decoding.  If yes, then that will slow us down */
         for (k = 0; k < src_num_comp; k++) {
@@ -127,9 +133,12 @@ gs_image_class_4_color(gx_image_enum * penum)
             }
         }
         /* Define the rendering intents */
-        rendering_params.black_point_comp = BP_ON;
+        rendering_params.black_point_comp = penum->pis->blackptcomp;
         rendering_params.graphics_type_tag = GS_IMAGE_TAG;
+        rendering_params.override_icc = false;
+        rendering_params.preserve_black = gsBKPRESNOTSPECIFIED;
         rendering_params.rendering_intent = penum->pis->renderingintent;
+        rendering_params.cmm = gsCMM_DEFAULT;
         if (gs_color_space_is_PSCIE(penum->pcs) && penum->pcs->icc_equivalent != NULL) {
             pcs = penum->pcs->icc_equivalent;
         } else {
@@ -142,19 +151,20 @@ gs_image_class_4_color(gx_image_enum * penum)
         if (penum->icc_setup.is_lab) penum->icc_setup.need_decode = false;
         if (penum->icc_link == NULL) {
             penum->icc_link = gsicc_get_link(penum->pis, penum->dev, pcs, NULL,
-                &rendering_params, penum->memory, false);
+                &rendering_params, penum->memory);
         }
         /* PS CIE color spaces may have addition decoding that needs to
            be performed to ensure that the range of 0 to 1 is provided
            to the CMM since ICC profiles are restricted to that range
            but the PS color spaces are not. */
+        penum->use_cie_range = false;
         if (gs_color_space_is_PSCIE(penum->pcs) &&
             penum->pcs->icc_equivalent != NULL) {
             /* We have a PS CIE space.  Check the range */
             if ( !check_cie_range(penum->pcs) ) {
                 /* It is not 0 to 1.  We will be doing decode
                    plus an additional linear adjustment */
-                penum->cie_range = get_cie_range(penum->pcs);
+                penum->use_cie_range = (get_cie_range(penum->pcs) != NULL);
             }
         }
         if (gx_device_must_halftone(penum->dev) && use_fast_thresh &&
@@ -164,8 +174,8 @@ gs_image_class_4_color(gx_image_enum * penum)
             /* If num components is 1 or if we are going to CMYK planar device
                then we will may use the thresholding if it is a halftone
                device*/
-            is_planar_dev = dev_proc(penum->dev, dev_spec_op)(penum->dev,
-                                 gxdso_is_native_planar, NULL, 0);
+            is_planar_dev = (dev_proc(penum->dev, dev_spec_op)(penum->dev,
+                                 gxdso_is_native_planar, NULL, 0) > 0);
             if ((penum->dev->color_info.num_components == 1 || is_planar_dev) &&
                  penum->bps == 8 ) {
                 code = gxht_thresh_image_init(penum);
@@ -283,12 +293,10 @@ decode_row(const gx_image_enum *penum, const byte *psrc, int spp, byte *pdes,
 static int
 image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
                      gx_device *dev, int *spp_cm_out, byte **psrc_cm,
-                     byte **psrc_cm_start, byte **psrc_decode, byte **bufend,
-                     bool planar_out)
+                     byte **psrc_cm_start, byte **bufend, bool planar_out)
 {
     const gx_image_enum *const penum = penum_orig; /* const within proc */
     const gs_imager_state *pis = penum->pis;
-    const gs_color_space *pcs;
     bool need_decode = penum->icc_setup.need_decode;
     gsicc_bufferdesc_t input_buff_desc;
     gsicc_bufferdesc_t output_buff_desc;
@@ -298,17 +306,18 @@ image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
     int num_des_comps;
     int code;
     cmm_dev_profile_t *dev_profile;
+    byte *psrc_decode;
+    const byte *planar_src;
+    byte *planar_des;
+    int k, j;
+    int width;
 
     code = dev_proc(dev, get_profile)(dev, &dev_profile);
-    num_des_comps = dev_profile->device_profile[0]->num_comps;
+    if (code < 0) return code;
+    num_des_comps = gsicc_get_device_profile_comps(dev_profile);
     if (penum->icc_link == NULL) {
         return gs_rethrow(-1, "ICC Link not created during image render color");
     }
-    if (gs_color_space_is_PSCIE(penum->pcs) && penum->pcs->icc_equivalent != NULL) {
-        pcs = penum->pcs->icc_equivalent;
-    } else {
-        pcs = penum->pcs;
-    }
     /* If the link is the identity, then we don't need to do any color
        conversions except for potentially a decode.  Planar out is a special
        case. For now we let the CMM do the reorg into planar.  We will want
@@ -326,12 +335,48 @@ image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
     } else {
         spp_cm = num_des_comps;
         *psrc_cm = gs_alloc_bytes(pis->memory,  w * spp_cm/spp,
-                                  "image_render_color_icc");
+                                  "image_color_icc_prep");
         *psrc_cm_start = *psrc_cm;
         *bufend = *psrc_cm +  w * spp_cm/spp;
-        if (penum->icc_link->is_identity && !force_planar) {
-            /* decode only. no CM.  This is slow but does not happen that often */
-            decode_row(penum, psrc, spp, *psrc_cm, *bufend);
+        if (penum->icc_link->is_identity) {
+            if (!force_planar) {
+                /* decode only. no CM.  This is slow but does not happen that often */
+                decode_row(penum, psrc, spp, *psrc_cm, *bufend);
+            } else {
+                /* CM is identity but we may need to do decode and then off 
+                   to planar. The planar out case is only used when coming from
+                   imager_render_color_thresh, which is limited to 8 bit case */
+                if (need_decode) {
+                    /* Need decode and then to planar */
+                    psrc_decode = gs_alloc_bytes(pis->memory,  w,
+                                                  "image_color_icc_prep");
+                    if (!penum->use_cie_range) {
+                        decode_row(penum, psrc, spp, psrc_decode, psrc_decode+w);
+                    } else {
+                        /* Decode needs to include adjustment for CIE range */
+                        decode_row_cie(penum, psrc, spp, psrc_decode,
+                                        psrc_decode + w, get_cie_range(penum->pcs));
+                    }
+                    planar_src = psrc_decode;
+                } else {
+                    psrc_decode = NULL;
+                    planar_src = psrc;
+                }
+                /* Now to planar */
+                width = w/spp;
+                planar_des = *psrc_cm;
+                for (k = 0; k < width; k++) {
+                    for (j = 0; j < spp; j++) {
+                        *(planar_des + j * width) = *planar_src++; 
+                    }
+                    planar_des++;
+                }
+                /* Free up decode if we used it */
+                if (psrc_decode != NULL) {
+                    gs_free_object(pis->memory, (byte *) psrc_decode,
+                                   "image_render_color_icc");
+                }
+            }
         } else {
             /* Set up the buffer descriptors. planar out always ends up here */
             num_pixels = w/spp;
@@ -354,25 +399,28 @@ image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
                less than or equal to the new one.  */
             if (need_decode) {
                 /* Need decode and CM.  This is slow but does not happen that often */
-                *psrc_decode = gs_alloc_bytes(pis->memory,  w,
-                                              "image_render_color_icc");
-                if (penum->cie_range == NULL) {
-                    decode_row(penum, psrc, spp, *psrc_decode, (*psrc_decode)+w);
+                psrc_decode = gs_alloc_bytes(pis->memory, w, 
+                                              "image_color_icc_prep");
+                if (!penum->use_cie_range) {
+                    decode_row(penum, psrc, spp, psrc_decode, psrc_decode+w);
                 } else {
                     /* Decode needs to include adjustment for CIE range */
-                    decode_row_cie(penum, psrc, spp, *psrc_decode,
-                                    (*psrc_decode)+w, penum->cie_range);
+                    decode_row_cie(penum, psrc, spp, psrc_decode,
+                                    psrc_decode+w, get_cie_range(penum->pcs));
                 }
-                gscms_transform_color_buffer(penum->icc_link, &input_buff_desc,
-                                        &output_buff_desc, (void*) *psrc_decode,
-                                        (void*) *psrc_cm);
-                gs_free_object(pis->memory, (byte *) *psrc_decode,
-                               "image_render_color_icc");
+                (penum->icc_link->procs.map_buffer)(dev, penum->icc_link, 
+                                                    &input_buff_desc,
+                                                    &output_buff_desc, 
+                                                    (void*) psrc_decode,
+                                                    (void*) *psrc_cm);
+                gs_free_object(pis->memory, psrc_decode, "image_color_icc_prep");
             } else {
                 /* CM only. No decode */
-                gscms_transform_color_buffer(penum->icc_link, &input_buff_desc,
-                                            &output_buff_desc, (void*) psrc,
-                                            (void*) *psrc_cm);
+                (penum->icc_link->procs.map_buffer)(dev, penum->icc_link, 
+                                                    &input_buff_desc,
+                                                    &output_buff_desc, 
+                                                    (void*) psrc, 
+                                                    (void*) *psrc_cm);
             }
         }
     }
@@ -391,12 +439,10 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
     byte *thresh_align;
     byte *devc_contone[GX_DEVICE_COLOR_MAX_COMPONENTS];
     byte *psrc_plane[GX_DEVICE_COLOR_MAX_COMPONENTS];
-    byte *contone_align;
     byte *devc_contone_gray;
     const byte *psrc = buffer + data_x;
     int dest_width, dest_height, data_length;
     int spp_out = dev->color_info.num_components;
-    gx_ht_order *d_order;
     int position, k, j;
     int offset_bits = penum->ht_offset_bits;
     int contone_stride = 0;  /* Not used in landscape case */
@@ -409,18 +455,16 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
     gx_dda_int_t dda_ht;
     int code = 0;
     int spp_cm = 0;
-    byte *psrc_cm = NULL, *psrc_cm_start = NULL, *psrc_decode = NULL;
+    byte *psrc_cm = NULL, *psrc_cm_start = NULL;
     byte *bufend = NULL, *curr_ptr;
     int psrc_planestride = w/penum->spp;
     gx_color_value conc;
     int num_des_comp = penum->dev->color_info.num_components;
-    bool allow_reset;
 
     if (h != 0) {
         /* Get the buffer into the device color space */
         code = image_color_icc_prep(penum, psrc, w, dev, &spp_cm, &psrc_cm,
-                                    &psrc_cm_start, &psrc_decode, &bufend,
-                                    true);
+                                    &psrc_cm_start,  &bufend, true);
         /* Also, if need apply the transfer function at this time.  This
            should be reworked so that we are not doing all these conversions */
         if (penum->icc_setup.has_transfer) {
@@ -447,7 +491,7 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
     /* Data is now in the proper destination color space.  Now we want
        to go ahead and get the data into the proper spatial setting and then
        threshold.  First get the data spatially sampled correctly */
-    src_size = (penum->rect.w - 1.0);
+    src_size = penum->rect.w;
     switch (posture) {
         case image_portrait:
             /* Figure out our offset in the contone and threshold data
@@ -470,10 +514,10 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
                 xrun += penum->x_extent.x;
             data_length = dest_width;
             dest_height = fixed2int_var_rounded(any_abs(penum->y_extent.y));
-            scale_factor = float2fixed_rounded((float) src_size / (float) (dest_width - 1));
+            scale_factor = float2fixed_rounded((float) src_size / (float) dest_width);
 #ifdef DEBUG
             /* Help in spotting problems */
-            memset(penum->ht_buffer, 0x00, penum->ht_stride * vdi);
+            memset(penum->ht_buffer, 0x00, penum->ht_stride * vdi * spp_out);
 #endif
             break;
         case image_landscape:
@@ -492,7 +536,7 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
             dest_width = fixed2int_var_rounded(any_abs(penum->y_extent.x));
             dest_height = fixed2int_var_rounded(any_abs(penum->x_extent.y));
             data_length = dest_height;
-            scale_factor = float2fixed_rounded((float) src_size / (float) (dest_height - 1));
+            scale_factor = float2fixed_rounded((float) src_size / (float) dest_height);
             /* In the landscaped case, we want to accumulate multiple columns
                of data before sending to the device.  We want to have a full
                byte of HT data in one write.  This may not be possible at the
@@ -510,10 +554,13 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
                 if (penum->ht_landscape.index < 0) {
                     penum->ht_landscape.xstart = penum->xci + vdi - 1;
                     offset_bits = (penum->ht_landscape.xstart % 16) + 1;
+                    /* xci can be negative, so allow for that */
+                    if (offset_bits <= 0) offset_bits += 16;
                 } else {
                     penum->ht_landscape.xstart = penum->xci;
+                    /* xci can be negative, see Bug 692569. */
                     offset_bits = 16 - penum->xci % 16;
-                    if (offset_bits == 16) offset_bits = 0;
+                    if (offset_bits >= 16) offset_bits -= 16;
                 }
                 if (offset_bits == 0 || offset_bits == 16) {
                     penum->ht_landscape.offset_set = false;
@@ -527,7 +574,7 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
     /* Get the pointers to our buffers */
     if (flush_buff) goto flush;  /* All done */
     /* Set up the dda.  We could move this out but the cost is pretty small */
-    dda_init(dda_ht, 0, src_size, data_length-1);
+    dda_init_half(dda_ht, 0, src_size, data_length);
     /* Do conversion to device resolution in quick small loops. */
     /* For now we have 3 cases.  A CMYK (4 channel), gray, or other case
        the latter of which is not yet implemented */
@@ -550,9 +597,9 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
                 /* Monochrome portrait */
                 case image_portrait:
                     if (penum->dst_width > 0) {
-                        if (scale_factor == fixed_1) {
+                        if (src_size == dest_width) {
                             memcpy(devc_contone_gray, psrc_cm, data_length);
-                        } else if (scale_factor == fixed_half) {
+                        } else if (src_size * 2 == dest_width) {
                             psrc_temp = psrc_cm;
                             for (k = 0; k < data_length; k+=2,
                                  devc_contone_gray+=2, psrc_temp++) {
@@ -592,7 +639,7 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
                         /* Code up special cases for when we have no scaling
                            and 2x scaling which we will run into in 300 and
                            600dpi devices and content */
-                        if (scale_factor == fixed_1) {
+                        if (src_size == dest_height) {
                             for (k = 0; k < data_length; k++) {
                                 devc_contone_gray[position] = psrc_cm[k];
                                 position += LAND_BITS;
@@ -633,12 +680,12 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
                 /* CMYK portrait */
                 case image_portrait:
                     if (penum->dst_width > 0) {
-                        if (scale_factor == fixed_1) {
+                        if (src_size == dest_width) {
                             memcpy(devc_contone[0], psrc_plane[0], data_length);
                             memcpy(devc_contone[1], psrc_plane[1], data_length);
                             memcpy(devc_contone[2], psrc_plane[2], data_length);
                             memcpy(devc_contone[3], psrc_plane[3], data_length);
-                        } else if (scale_factor == fixed_half) {
+                        } else if (src_size * 2 == dest_width) {
                             for (k = 0; k < data_length; k+=2) {
                                 *(devc_contone[0]) = *(devc_contone[0]+1) =
                                     *psrc_plane[0]++;
@@ -706,7 +753,7 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
                         for (k = 0; k < spp_out; k++) {
                             devc_contone[k] = devc_contone[k] + position;
                         }
-                        if (scale_factor == fixed_1) {
+                        if (src_size == dest_height) {
                             for (k = 0; k < data_length; k++) {
                                 /* Is it better to unwind this?  We know it is 4 */
                                 for (j = 0; j < spp_out; j++) {
@@ -757,25 +804,9 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
        depnding upon the polarity of the device */
 flush:
     thresh_align = penum->thresh_buffer + offset_threshold;
-    for (k = 0; k < spp_out; k++) {
-        d_order = &(penum->pis->dev_ht->components[k].corder);
-        if (posture == image_portrait) {
-            contone_align = penum->line + contone_stride * k + 
-                            offset_contone[k];
-            allow_reset = true;
-        } else {
-            contone_align = penum->line + offset_contone[k] +
-                              LAND_BITS * k * contone_stride;
-            if (k == spp_out - 1) {
-                allow_reset = true;
-            } else {
-                allow_reset = false;
-            }
-        }
-        code = gxht_thresh_plane(penum, d_order, xrun, dest_width, dest_height,
-                                 thresh_align, contone_align, 
-                                 contone_stride, dev, k, allow_reset);
-    }
+    code = gxht_thresh_planes(penum, xrun, dest_width, dest_height,
+                              thresh_align, dev, offset_contone,
+                               contone_stride);
     /* Free cm buffer, if it was used */
     if (psrc_cm_start != NULL) {
         gs_free_object(penum->pis->memory, (byte *)psrc_cm_start,
@@ -812,8 +843,8 @@ image_render_color_icc(gx_image_enum *penum_orig, const byte *buffer, int data_x
     color_samples run;		/* run value */
     color_samples next;		/* next sample value */
     byte *bufend = NULL;
-    int code = 0, mcode = 0;
-    byte *psrc_cm = NULL, *psrc_cm_start = NULL, *psrc_decode = NULL;
+    int code = 0;
+    byte *psrc_cm = NULL, *psrc_cm_start = NULL;
     int k;
     gx_color_value conc[GX_DEVICE_COLOR_MAX_COMPONENTS];
     int spp_cm = 0;
@@ -829,7 +860,7 @@ image_render_color_icc(gx_image_enum *penum_orig, const byte *buffer, int data_x
     if (h == 0)
         return 0;
     code = image_color_icc_prep(penum_orig, psrc, w, dev, &spp_cm, &psrc_cm,
-                                &psrc_cm_start, &psrc_decode, &bufend, false);
+                                &psrc_cm_start, &bufend, false);
     if (code < 0) return code;
     /* Needed for device N */
     memset(&(conc[0]), 0, sizeof(gx_color_value[GX_DEVICE_COLOR_MAX_COMPONENTS]));
@@ -849,8 +880,8 @@ image_render_color_icc(gx_image_enum *penum_orig, const byte *buffer, int data_x
             irun = fixed2int_var_rounded(yrun);
             break;
     }
-    if_debug5('b', "[b]y=%d data_x=%d w=%d xt=%f yt=%f\n",
-              penum->y, data_x, w, fixed2float(xprev), fixed2float(yprev));
+    if_debug5m('b', penum->memory, "[b]y=%d data_x=%d w=%d xt=%f yt=%f\n",
+               penum->y, data_x, w, fixed2float(xprev), fixed2float(yprev));
     memset(&run, 0, sizeof(run));
     memset(&next, 0, sizeof(next));
     run.v[0] = ~psrc_cm[0];	/* Force intial setting */
@@ -868,7 +899,7 @@ image_render_color_icc(gx_image_enum *penum_orig, const byte *buffer, int data_x
         if (posture != image_skewed && next.all[0] == run.all[0])
                 goto inc;
         /* This needs to be sped up */
-         for ( k = 0; k < spp_cm; k++ ) {
+        for ( k = 0; k < spp_cm; k++ ) {
             conc[k] = gx_color_value_from_byte(next.v[k]);
         }
         /* Now we can do an encoding directly or we have to apply transfer
@@ -930,7 +961,6 @@ image_render_color_icc(gx_image_enum *penum_orig, const byte *buffer, int data_x
         if (code < 0)
             goto err;
         rsrc = psrc;
-        if ((code = mcode) < 0) goto err;
         /* Swap around the colors due to a change */
         ptemp = pdevc;
         pdevc = pdevc_next;
@@ -1025,14 +1055,32 @@ image_render_color_DeviceN(gx_image_enum *penum_orig, const byte *buffer, int da
     color_samples next;		/* next sample value */
     const byte *bufend = psrc + w;
     int code = 0, mcode = 0;
-            int i;
+    int i;
     bits32 mask = penum->mask_color.mask;
     bits32 test = penum->mask_color.test;
+    bool lab_case = false;
 
     if (h == 0)
         return 0;
+
+    /* Decide on which remap proc to use.  If the source colors are LAB
+       then use the mapping that does not rescale the source colors */
+    if (gs_color_space_is_ICC(pcs) && pcs->cmm_icc_profile_data != NULL &&
+        pcs->cmm_icc_profile_data->islab) {
+        remap_color = gx_remap_ICC_imagelab;
+        lab_case = true;
+    } else {
+        remap_color = pcs->type->remap_color;
+    }
     pdevc = &devc1;
     pdevc_next = &devc2;
+    /* In case these are devn colors */
+    if (dev_proc(dev, dev_spec_op)(dev, gxdso_supports_devn, NULL, 0)) {
+        for (i = 0; i < GS_CLIENT_COLOR_MAX_COMPONENTS; i++) {
+            pdevc->colors.devn.values[i] = 0;
+            pdevc_next->colors.devn.values[i] = 0;
+        }
+    }
     /* These used to be set by init clues */
     pdevc->type = gx_dc_type_none;
     pdevc_next->type = gx_dc_type_none;
@@ -1052,8 +1100,8 @@ image_render_color_DeviceN(gx_image_enum *penum_orig, const byte *buffer, int da
             irun = fixed2int_var_rounded(yrun);
             break;
     }
-    if_debug5('b', "[b]y=%d data_x=%d w=%d xt=%f yt=%f\n",
-              penum->y, data_x, w, fixed2float(xprev), fixed2float(yprev));
+    if_debug5m('b', penum->memory, "[b]y=%d data_x=%d w=%d xt=%f yt=%f\n",
+               penum->y, data_x, w, fixed2float(xprev), fixed2float(yprev));
     memset(&run, 0, sizeof(run));
     memset(&next, 0, sizeof(next));
     cs_full_init_color(&cc, pcs);
@@ -1074,15 +1122,20 @@ image_render_color_DeviceN(gx_image_enum *penum_orig, const byte *buffer, int da
             color_set_null(pdevc_next);
             goto mapped;
         }
-        for (i = 0; i < spp; ++i)
-            decode_sample(next.v[i], cc, i);
+        /* Data is already properly set up for ICC use of LAB */
+        if (lab_case)
+            for (i = 0; i < spp; ++i)
+                cc.paint.values[i] = (next.v[i]) * (1.0f / 255.0f); 
+        else
+            for (i = 0; i < spp; ++i)
+                decode_sample(next.v[i], cc, i);
 #ifdef DEBUG
         if (gs_debug_c('B')) {
-            dprintf2("[B]cc[0..%d]=%g", spp - 1,
+            dmprintf2(dev->memory, "[B]cc[0..%d]=%g", spp - 1,
                      cc.paint.values[0]);
             for (i = 1; i < spp; ++i)
-                dprintf1(",%g", cc.paint.values[i]);
-            dputs("\n");
+                dmprintf1(dev->memory, ",%g", cc.paint.values[i]);
+            dmputs(dev->memory, "\n");
         }
 #endif
         mcode = remap_color(&cc, pcs, pdevc_next, pis, dev,
@@ -1090,21 +1143,23 @@ image_render_color_DeviceN(gx_image_enum *penum_orig, const byte *buffer, int da
 mapped:	if (mcode < 0)
             goto fill;
         if (sizeof(pdevc_next->colors.binary.color[0]) <= sizeof(ulong))
-            if_debug7('B', "[B]0x%x,0x%x,0x%x,0x%x -> 0x%lx,0x%lx,0x%lx\n",
-                  next.v[0], next.v[1], next.v[2], next.v[3],
-                  (ulong)pdevc_next->colors.binary.color[0],
-                  (ulong)pdevc_next->colors.binary.color[1],
-                  (ulong) pdevc_next->type);
+            if_debug7m('B', penum->memory,
+                       "[B]0x%x,0x%x,0x%x,0x%x -> 0x%lx,0x%lx,0x%lx\n",
+                       next.v[0], next.v[1], next.v[2], next.v[3],
+                       (ulong)pdevc_next->colors.binary.color[0],
+                       (ulong)pdevc_next->colors.binary.color[1],
+                       (ulong) pdevc_next->type);
         else
-            if_debug9('B', "[B]0x%x,0x%x,0x%x,0x%x -> 0x%08lx%08lx,0x%08lx%08lx,0x%lx\n",
-                  next.v[0], next.v[1], next.v[2], next.v[3],
-                  (ulong)(pdevc_next->colors.binary.color[0] >>
-                        8 * (sizeof(pdevc_next->colors.binary.color[0]) - sizeof(ulong))),
-                  (ulong)pdevc_next->colors.binary.color[0],
-                  (ulong)(pdevc_next->colors.binary.color[1] >>
-                        8 * (sizeof(pdevc_next->colors.binary.color[1]) - sizeof(ulong))),
-                  (ulong)pdevc_next->colors.binary.color[1],
-                  (ulong) pdevc_next->type);
+            if_debug9m('B', penum->memory,
+                       "[B]0x%x,0x%x,0x%x,0x%x -> 0x%08lx%08lx,0x%08lx%08lx,0x%lx\n",
+                       next.v[0], next.v[1], next.v[2], next.v[3],
+                       (ulong)(pdevc_next->colors.binary.color[0] >>
+                               8 * (sizeof(pdevc_next->colors.binary.color[0]) - sizeof(ulong))),
+                       (ulong)pdevc_next->colors.binary.color[0],
+                       (ulong)(pdevc_next->colors.binary.color[1] >>
+                               8 * (sizeof(pdevc_next->colors.binary.color[1]) - sizeof(ulong))),
+                       (ulong)pdevc_next->colors.binary.color[1],
+                       (ulong) pdevc_next->type);
         /* NB: printf above fails to account for sizeof gx_color_index 4 or 8 bytes */
         if (posture != image_skewed && dev_color_eq(*pdevc, *pdevc_next))
             goto set;
