@@ -1,33 +1,28 @@
-
-/* Copyright (C) 2011 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
+
 /* opj filter implementation using OpenJPeg library */
 
 #include "memory_.h"
-#include "malloc_.h"
 #include "gserrors.h"
 #include "gdebug.h"
 #include "strimpl.h"
 #include "sjpx_openjpeg.h"
 
-
-/* As with the /JBIG2Decode filter, we let the library do its
-   memory management through malloc() etc. and rely on our release()
-   proc being called to deallocate state.
-*/
-
 gs_private_st_simple(st_jpxd_state, stream_jpxd_state,
     "JPXDecode filter state"); /* creates a gc object for our state,
                             defined in sjpx.h */
@@ -44,10 +39,6 @@ s_opjd_init(stream_state * ss)
     stream_jpxd_state *const state = (stream_jpxd_state *) ss;
 	opj_dparameters_t parameters;	/* decompression parameters */
 
-    if (state->jpx_memory == NULL) {
-        state->jpx_memory = ss->memory->non_gc_memory;
-    }
-
     /* get a decoder handle */
     state->opj_dinfo_p = opj_create_decompress(CODEC_JP2);
 
@@ -55,7 +46,7 @@ s_opjd_init(stream_state * ss)
             return_error(gs_error_VMerror);
 
     /* catch events using our callbacks and give a local context */
-    //opj_set_event_mgr((opj_common_ptr)dinfo, &event_mgr, stderr);
+ /* opj_set_event_mgr((opj_common_ptr)dinfo, &event_mgr, stderr); */
 
     /* set decoding parameters to default values */
     opj_set_default_decoder_parameters(&parameters);
@@ -103,7 +94,7 @@ s_jpxd_ycc_to_rgb(stream_jpxd_state *state)
         sgnd[i] = state->image->comps[i+1].sgnd;
 
     for (i=0; i<3; i++)
-        row_bufs[i] = malloc(sizeof(int)*state->width);
+    	row_bufs[i] = (int *)gs_alloc_byte_array(state->memory->non_gc_memory, sizeof(int)*state->width, 1, "s_jpxd_ycc_to_rgb");
 
     if (!row_bufs[0] || !row_bufs[1] || !row_bufs[2])
         return_error(gs_error_VMerror);
@@ -150,7 +141,7 @@ s_jpxd_ycc_to_rgb(stream_jpxd_state *state)
     }
 
     for (i=0; i<3; i++)
-        free(row_bufs[i]);
+    	gs_free_object(state->memory->non_gc_memory, row_bufs[i],"s_jpxd_ycc_to_rgb");
 
     return 0;
 }
@@ -158,7 +149,7 @@ s_jpxd_ycc_to_rgb(stream_jpxd_state *state)
 static int decode_image(stream_jpxd_state * const state)
 {
     opj_cio_t *cio = NULL;
-    int numprimcomp = 0, alpha_comp = -1, compno;
+    int numprimcomp = 0, alpha_comp = -1, compno, rowbytes;
 
     /* open a byte stream */
     cio = opj_cio_open((opj_common_ptr)state->opj_dinfo_p, state->inbuf, state->inbuf_fill);
@@ -188,7 +179,7 @@ static int decode_image(stream_jpxd_state * const state)
     for(compno = 1; compno < state->image->numcomps; compno++)
     {
         if (state->bpp != state->image->comps[compno].prec)
-            return ERRC; // Not supported.
+            return ERRC; /* Not supported. */
         if (state->width < state->image->comps[compno].w)
             state->width = state->image->comps[compno].w;
         if (state->height < state->image->comps[compno].h)
@@ -246,23 +237,24 @@ static int decode_image(stream_jpxd_state * const state)
     else
         state->out_numcomps = numprimcomp;
 
-    /* round up bpp to multiple of 8, eg 12->16 */
-    if ((state->colorspace != gs_jpx_cs_gray && !state->image->has_palette) || state->bpp>8)
-        state->bpp = (state->bpp+7)/8*8;
+    /* round up bpp 12->16 */
+    if (state->bpp == 12)
+        state->bpp = 16;
 
     /* calculate  total data */
-    state->totalbytes = state->width*state->height*state->bpp*state->out_numcomps/8;
+    rowbytes =  (state->width*state->bpp*state->out_numcomps+7)/8;
+    state->totalbytes = rowbytes*state->height;
 
     /* convert input from YCC to RGB */
     if (state->image->color_space == CLRSPC_SYCC || state->image->color_space == CLRSPC_EYCC)
         s_jpxd_ycc_to_rgb(state);
 
-    state->pdata = malloc(sizeof(int*)*state->image->numcomps);
+    state->pdata = (int **)gs_alloc_byte_array(state->memory->non_gc_memory, sizeof(int*)*state->image->numcomps, 1, "decode_image(pdata)");
     if (!state->pdata)
         return_error(gs_error_VMerror);
 
     /* compensate for signed data (signed => unsigned) */
-    state->sign_comps = malloc(sizeof(int)*state->image->numcomps);
+    state->sign_comps = (int *)gs_alloc_byte_array(state->memory->non_gc_memory, sizeof(int)*state->image->numcomps, 1, "decode_image(sign_comps)");
     if (!state->sign_comps)
         return_error(gs_error_VMerror);
 
@@ -282,7 +274,7 @@ static int process_one_trunk(stream_jpxd_state * const state, stream_cursor_writ
      /* read data from image to pw */
      unsigned long out_size = pw->limit - pw->ptr;
      int bytepp1 = state->bpp/8; /* bytes / pixel for one output component */
-     int bytepp = state->out_numcomps*state->bpp/8; /* bytes / pixel all compoments */
+     int bytepp = state->out_numcomps*state->bpp/8; /* bytes / pixel all components */
      unsigned long write_size = min(out_size-(bytepp?(out_size%bytepp):0), state->totalbytes-state->out_offset);
      unsigned long in_offset = state->out_offset*8/state->bpp/state->out_numcomps; /* component data offset */
      int shift_bit = state->bpp-state->image->comps[0].prec; /*difference between input and output bit-depth*/
@@ -335,21 +327,55 @@ static int process_one_trunk(stream_jpxd_state * const state, stream_cursor_writ
              }
              else
              {   
-                 /* only grayscale can have such bit-depth, also shift_bit = 0, bpp < 8 */
+                 /* shift_bit = 0, bpp < 8 */
                  unsigned long image_total = state->width*state->height;
+                 int bt=0; int bit_pos = 0;
+                 int rowbytes =  (state->width*state->bpp*state->out_numcomps+7)/8; /*row bytes */
+                 int currowcnt = state->out_offset % rowbytes; /* number of bytes filled in current row */
+                 int start_comp = (currowcnt*8) % img_numcomps; /* starting component for this round of output*/
+                 if (start_comp != 0)
+                 {
+                     for (compno=start_comp; compno<img_numcomps; compno++)
+                     {
+                         if (state->img_offset < image_total)
+                         {
+                             bt <<= state->bpp;
+                             bt += *(state->pdata[compno]-1) + state->sign_comps[compno];
+                         }
+                         bit_pos += state->bpp;
+                         if (bit_pos >= 8)
+                         {
+                             *(pw->ptr++) = bt >> (bit_pos-8);
+                             bit_pos -= 8;
+                             bt &= (1<<bit_pos)-1;
+                         }
+                     }
+                 }
                  while (pw->ptr < pend)
                  {
-                     int bt=0;
-                     for (i=0; i<8/state->bpp; i++)
+                     for (compno=0; compno<img_numcomps; compno++)
                      {
-                         bt = bt<<state->bpp;
-                         if (state->img_offset < image_total && !(i!=0 && state->img_offset % state->width == 0))
+                         if (state->img_offset < image_total)
                          {
-                             bt += *(state->pdata[0]++) + state->sign_comps[0];
-                             state->img_offset++;
+                             bt <<= state->bpp;
+                             bt += *(state->pdata[compno]++) + state->sign_comps[compno];
                          }
-                      }
-                      *(pw->ptr++) = bt;
+                         bit_pos += state->bpp;
+                         if (bit_pos >= 8)
+                         {
+                             *(pw->ptr++) = bt >> (bit_pos-8);
+                             bit_pos -= 8;
+                             bt &= (1<<bit_pos)-1;
+                         }
+                     }
+                     state->img_offset++;
+                     if (bit_pos != 0 && state->img_offset % state->width == 0)
+                     {
+                         /* row padding */
+                         *(pw->ptr++) = bt << (8 - bit_pos);
+                         bit_pos = 0;
+                         bt = 0;
+                     }
                  }
              }
          }
@@ -485,13 +511,13 @@ s_opjd_release(stream_state *ss)
 
     /* free input buffer */
     if (state->inbuf)
-	free(state->inbuf);
+        gs_free_object(state->memory->non_gc_memory, state->inbuf, "s_opjd_release(inbuf)");
 
     if (state->pdata)
-	free(state->pdata);
+        gs_free_object(state->memory->non_gc_memory, state->pdata, "s_opjd_release(pdata)");
 
     if (state->sign_comps)
-	free(state->sign_comps);
+        gs_free_object(state->memory->non_gc_memory, state->sign_comps, "s_opjd_release(sign_comps)");
 }
 
 
@@ -511,10 +537,10 @@ s_opjd_accumulate_input(stream_jpxd_state *state, stream_cursor_read * pr)
 
         if_debug1('s', "[s]opj growing input buffer to %lu bytes\n",
                 new_size);
-	if (state->inbuf == NULL)
-            new_buf = (byte *) malloc(new_size);
-	else
-            new_buf = (byte *) realloc(state->inbuf, new_size);
+        if (state->inbuf == NULL)
+            new_buf = (byte *) gs_alloc_byte_array(state->memory->non_gc_memory, new_size, 1, "s_opjd_accumulate_input(alloc)");
+        else
+            new_buf = (byte *) gs_resize_object(state->memory->non_gc_memory, state->inbuf, new_size, "s_opjd_accumulate_input(resize)");
         if (new_buf == NULL) return_error( gs_error_VMerror);
 
         state->inbuf = new_buf;
