@@ -5,6 +5,7 @@
  * Copyright (c) 2002-2003, Yannick Verschueren
  * Copyright (c) 2003-2007, Francois-Olivier Devaux and Antonin Descampe
  * Copyright (c) 2005, Herve Drolon, FreeImage Team
+ * Copyright (c) 2010-2011, Kaori Hagihara
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -43,7 +44,7 @@ Read box headers
 @param box
 @return Returns true if successful, returns false otherwise
 */
-static bool jp2_read_boxhdr(opj_common_ptr cinfo, opj_cio_t *cio, opj_jp2_box_t *box);
+static opj_bool jp2_read_boxhdr(opj_common_ptr cinfo, opj_cio_t *cio, opj_jp2_box_t *box);
 /*static void jp2_write_url(opj_cio_t *cio, char *Idx_file);*/
 /**
 Read the IHDR box - Image Header box
@@ -51,10 +52,10 @@ Read the IHDR box - Image Header box
 @param cio Input buffer stream
 @return Returns true if successful, returns false otherwise
 */
-static bool jp2_read_ihdr(opj_jp2_t *jp2, opj_cio_t *cio);
+static opj_bool jp2_read_ihdr(opj_jp2_t *jp2, opj_cio_t *cio);
 static void jp2_write_ihdr(opj_jp2_t *jp2, opj_cio_t *cio);
 static void jp2_write_bpcc(opj_jp2_t *jp2, opj_cio_t *cio);
-static bool jp2_read_bpcc(opj_jp2_t *jp2, opj_cio_t *cio);
+static opj_bool jp2_read_bpcc(opj_jp2_t *jp2, opj_cio_t *cio);
 static void jp2_write_colr(opj_jp2_t *jp2, opj_cio_t *cio);
 /**
 Write the FTYP box - File type box
@@ -68,9 +69,9 @@ Read the FTYP box - File type box
 @param cio Input buffer stream
 @return Returns true if successful, returns false otherwise
 */
-static bool jp2_read_ftyp(opj_jp2_t *jp2, opj_cio_t *cio);
+static opj_bool jp2_read_ftyp(opj_jp2_t *jp2, opj_cio_t *cio);
 static int jp2_write_jp2c(opj_jp2_t *jp2, opj_cio_t *cio, opj_image_t *image, opj_codestream_info_t *cstr_info);
-static bool jp2_read_jp2c(opj_jp2_t *jp2, opj_cio_t *cio, unsigned int *j2k_codestream_length, unsigned int *j2k_codestream_offset);
+static opj_bool jp2_read_jp2c(opj_jp2_t *jp2, opj_cio_t *cio, unsigned int *j2k_codestream_length, unsigned int *j2k_codestream_offset);
 static void jp2_write_jp(opj_cio_t *cio);
 /**
 Read the JP box - JPEG 2000 signature
@@ -78,7 +79,7 @@ Read the JP box - JPEG 2000 signature
 @param cio Input buffer stream
 @return Returns true if successful, returns false otherwise
 */
-static bool jp2_read_jp(opj_jp2_t *jp2, opj_cio_t *cio);
+static opj_bool jp2_read_jp(opj_jp2_t *jp2, opj_cio_t *cio);
 /**
 Decode the structure of a JP2 file
 @param jp2 JP2 handle
@@ -86,7 +87,7 @@ Decode the structure of a JP2 file
 @param color Collector for profile, cdef and pclr data
 @return Returns true if successful, returns false otherwise
 */
-static bool jp2_read_struct(opj_jp2_t *jp2, opj_cio_t *cio,
+static opj_bool jp2_read_struct(opj_jp2_t *jp2, opj_cio_t *cio,
 	opj_jp2_color_t *color);
 /**
 Apply collected palette data
@@ -102,7 +103,7 @@ Collect palette data
 @param color Collector for profile, cdef and pclr data
 @return Returns true if successful, returns false otherwise
 */
-static bool jp2_read_pclr(opj_jp2_t *jp2, opj_cio_t *cio,
+static opj_bool jp2_read_pclr(opj_jp2_t *jp2, opj_cio_t *cio,
     opj_jp2_box_t *box, opj_jp2_color_t *color);
 /**
 Collect component mapping data
@@ -112,7 +113,7 @@ Collect component mapping data
 @param color Collector for profile, cdef and pclr data
 @return Returns true if successful, returns false otherwise
 */
-static bool jp2_read_cmap(opj_jp2_t *jp2, opj_cio_t *cio,
+static opj_bool jp2_read_cmap(opj_jp2_t *jp2, opj_cio_t *cio,
     opj_jp2_box_t *box, opj_jp2_color_t *color);
 /**
 Collect colour specification data
@@ -122,22 +123,48 @@ Collect colour specification data
 @param color Collector for profile, cdef and pclr data
 @return Returns true if successful, returns false otherwise
 */
-static bool jp2_read_colr(opj_jp2_t *jp2, opj_cio_t *cio,
+static opj_bool jp2_read_colr(opj_jp2_t *jp2, opj_cio_t *cio,
     opj_jp2_box_t *box, opj_jp2_color_t *color);
+/**
+Write file Index (superbox)
+@param[in] offset_jp2c offset of jp2c box
+@param[in] length_jp2c length of jp2c box
+@param[in] offset_idx  offset of cidx box
+@param[in] length_idx  length of cidx box
+@param[in] cio         file output handle
+@return                length of fidx box
+*/
+static int write_fidx( int offset_jp2c, int length_jp2c, int offset_idx, int length_idx, opj_cio_t *cio);
+/**
+Write index Finder box
+@param[in] offset offset of fidx box
+@param[in] length length of fidx box
+@param[in] cio         file output handle
+*/
+static void write_iptr( int offset, int length, opj_cio_t *cio);
+/**
+Write proxy box
+@param[in] offset_jp2c offset of jp2c box
+@param[in] length_jp2c length of jp2c box
+@param[in] offset_idx  offset of cidx box
+@param[in] length_idx  length of cidx box
+@param[in] cio         file output handle
+*/
+static void write_prxy( int offset_jp2c, int length_jp2c, int offset_idx, int length_idx, opj_cio_t *cio);
 /*@}*/
 
 /*@}*/
 
 /* ----------------------------------------------------------------------- */
 
-static bool jp2_read_boxhdr(opj_common_ptr cinfo, opj_cio_t *cio, opj_jp2_box_t *box) {
+static opj_bool jp2_read_boxhdr(opj_common_ptr cinfo, opj_cio_t *cio, opj_jp2_box_t *box) {
 	box->init_pos = cio_tell(cio);
 	box->length = cio_read(cio, 4);
 	box->type = cio_read(cio, 4);
 	if (box->length == 1) {
 		if (cio_read(cio, 4) != 0) {
 			opj_event_msg(cinfo, EVT_ERROR, "Cannot handle box sizes higher than 2^32\n");
-			return false;
+			return OPJ_FALSE;
 		}
 		box->length = cio_read(cio, 4);
 		if (box->length == 0) 
@@ -147,7 +174,7 @@ static bool jp2_read_boxhdr(opj_common_ptr cinfo, opj_cio_t *cio, opj_jp2_box_t
 		box->length = cio_numbytesleft(cio) + 8;
 	}
 	
-	return true;
+	return OPJ_TRUE;
 }
 
 #if 0
@@ -174,7 +201,7 @@ static void jp2_write_url(opj_cio_t *cio, char *Idx_file) {
 }
 #endif
 
-static bool jp2_read_ihdr(opj_jp2_t *jp2, opj_cio_t *cio) {
+static opj_bool jp2_read_ihdr(opj_jp2_t *jp2, opj_cio_t *cio) {
 	opj_jp2_box_t box;
 
 	opj_common_ptr cinfo = jp2->cinfo;
@@ -182,7 +209,7 @@ static bool jp2_read_ihdr(opj_jp2_t *jp2, opj_cio_t *cio) {
 	jp2_read_boxhdr(cinfo, cio, &box);
 	if (JP2_IHDR != box.type) {
 		opj_event_msg(cinfo, EVT_ERROR, "Expected IHDR Marker\n");
-		return false;
+		return OPJ_FALSE;
 	}
 
 	jp2->h = cio_read(cio, 4);			/* HEIGHT */
@@ -198,10 +225,10 @@ static bool jp2_read_ihdr(opj_jp2_t *jp2, opj_cio_t *cio) {
 
 	if (cio_tell(cio) - box.init_pos != box.length) {
 		opj_event_msg(cinfo, EVT_ERROR, "Error with IHDR Box\n");
-		return false;
+		return OPJ_FALSE;
 	}
 
-	return true;
+	return OPJ_TRUE;
 }
 
 static void jp2_write_ihdr(opj_jp2_t *jp2, opj_cio_t *cio) {
@@ -246,7 +273,7 @@ static void jp2_write_bpcc(opj_jp2_t *jp2, opj_cio_t *cio) {
 }
 
 
-static bool jp2_read_bpcc(opj_jp2_t *jp2, opj_cio_t *cio) {
+static opj_bool jp2_read_bpcc(opj_jp2_t *jp2, opj_cio_t *cio) {
 	unsigned int i;
 	opj_jp2_box_t box;
 
@@ -255,7 +282,7 @@ static bool jp2_read_bpcc(opj_jp2_t *jp2, opj_cio_t *cio) {
 	jp2_read_boxhdr(cinfo, cio, &box);
 	if (JP2_BPCC != box.type) {
 		opj_event_msg(cinfo, EVT_ERROR, "Expected BPCC Marker\n");
-		return false;
+		return OPJ_FALSE;
 	}
 
 	for (i = 0; i < jp2->numcomps; i++) {
@@ -264,10 +291,10 @@ static bool jp2_read_bpcc(opj_jp2_t *jp2, opj_cio_t *cio) {
 
 	if (cio_tell(cio) - box.init_pos != box.length) {
 		opj_event_msg(cinfo, EVT_ERROR, "Error with BPCC Box\n");
-		return false;
+		return OPJ_FALSE;
 	}
 
-	return true;
+	return OPJ_TRUE;
 }
 
 static void jp2_write_colr(opj_jp2_t *jp2, opj_cio_t *cio) {
@@ -281,11 +308,10 @@ static void jp2_write_colr(opj_jp2_t *jp2, opj_cio_t *cio) {
 	cio_write(cio, jp2->precedence, 1);	/* PRECEDENCE */
 	cio_write(cio, jp2->approx, 1);		/* APPROX */
 
-	if (jp2->meth == 1) {
-		cio_write(cio, jp2->enumcs, 4);	/* EnumCS */
-	} else {
-		cio_write(cio, 0, 1);			/* PROFILE (??) */
-	}
+	if(jp2->meth == 2)
+	 jp2->enumcs = 0;
+
+	cio_write(cio, jp2->enumcs, 4);	/* EnumCS */
 
 	box.length = cio_tell(cio) - box.init_pos;
 	cio_seek(cio, box.init_pos);
@@ -343,10 +369,13 @@ static void jp2_apply_pclr(opj_jp2_color_t *color, opj_image_t *image, opj_commo
    {
 	pcol = cmap[i].pcol; cmp = cmap[i].cmp;
 
-	 if( pcol < nr_channels )
-	    new_comps[pcol] = old_comps[cmp];
-	 else
-	    opj_event_msg(cinfo, EVT_ERROR, "Error with pcol value. skipping\n");
+  if( pcol < nr_channels )
+    new_comps[pcol] = old_comps[cmp];
+  else
+    {
+    opj_event_msg(cinfo, EVT_ERROR, "Error with pcol value %d (max: %d). skipping\n", pcol, nr_channels);
+    continue;
+    }
 
 	if(cmap[i].mtyp == 0) /* Direct use */
   {
@@ -392,7 +421,7 @@ static void jp2_apply_pclr(opj_jp2_color_t *color, opj_image_t *image, opj_commo
 
 }/* apply_pclr() */
 
-static bool jp2_read_pclr(opj_jp2_t *jp2, opj_cio_t *cio,
+static opj_bool jp2_read_pclr(opj_jp2_t *jp2, opj_cio_t *cio,
 	opj_jp2_box_t *box, opj_jp2_color_t *color)
 {
 	opj_jp2_pclr_t *jp2_pclr;
@@ -402,13 +431,16 @@ static bool jp2_read_pclr(opj_jp2_t *jp2, opj_cio_t *cio,
 	unsigned short i, j;
 	unsigned char uc;
 
+	OPJ_ARG_NOT_USED(box);
+	OPJ_ARG_NOT_USED(jp2);
+
 /* Part 1, I.5.3.4: 'There shall be at most one Palette box inside
  * a JP2 Header box' :
 */
-	if(color->jp2_pclr) return false;
+	if(color->jp2_pclr) return OPJ_FALSE;
 
-	nr_entries = cio_read(cio, 2); /* NE */
-	nr_channels = cio_read(cio, 1);/* NPC */
+	nr_entries = (unsigned short)cio_read(cio, 2); /* NE */
+	nr_channels = (unsigned short)cio_read(cio, 1);/* NPC */
 
 	entries = (unsigned int*)
 	 opj_malloc(nr_channels * nr_entries * sizeof(unsigned int));
@@ -441,22 +473,25 @@ static bool jp2_read_pclr(opj_jp2_t *jp2, opj_cio_t *cio,
   }
    }
 
-	return true;
+	return OPJ_TRUE;
 }/* jp2_read_pclr() */
 
-static bool jp2_read_cmap(opj_jp2_t *jp2, opj_cio_t *cio,
+static opj_bool jp2_read_cmap(opj_jp2_t *jp2, opj_cio_t *cio,
 	opj_jp2_box_t *box, opj_jp2_color_t *color)
 {
 	opj_jp2_cmap_comp_t *cmap;
 	unsigned short i, nr_channels;
 
+	OPJ_ARG_NOT_USED(box);
+	OPJ_ARG_NOT_USED(jp2);
+
 /* Need nr_channels: */
-	if(color->jp2_pclr == NULL) return false;
+	if(color->jp2_pclr == NULL) return OPJ_FALSE;
 
 /* Part 1, I.5.3.5: 'There shall be at most one Component Mapping box
  * inside a JP2 Header box' :
 */
-	if(color->jp2_pclr->cmap) return false;
+	if(color->jp2_pclr->cmap) return OPJ_FALSE;
 
 	nr_channels = color->jp2_pclr->nr_channels;
 	cmap = (opj_jp2_cmap_comp_t*)
@@ -464,14 +499,14 @@ static bool jp2_read_cmap(opj_jp2_t *jp2, opj_cio_t *cio,
 
 	for(i = 0; i < nr_channels; ++i)
    {
-	cmap[i].cmp = cio_read(cio, 2);
+	cmap[i].cmp = (unsigned short)cio_read(cio, 2);
 	cmap[i].mtyp = cio_read(cio, 1);
 	cmap[i].pcol = cio_read(cio, 1);
 
    }
 	color->jp2_pclr->cmap = cmap;
 
-	return true;
+	return OPJ_TRUE;
 }/* jp2_read_cmap() */
 
 static void jp2_apply_cdef(opj_image_t *image, opj_jp2_color_t *color)
@@ -514,18 +549,21 @@ static void jp2_apply_cdef(opj_image_t *image, opj_jp2_color_t *color)
 
 }/* jp2_apply_cdef() */
 
-static bool jp2_read_cdef(opj_jp2_t *jp2, opj_cio_t *cio,
+static opj_bool jp2_read_cdef(opj_jp2_t *jp2, opj_cio_t *cio,
 	opj_jp2_box_t *box, opj_jp2_color_t *color)
 {
 	opj_jp2_cdef_info_t *info;
 	unsigned short i, n;
 
+	OPJ_ARG_NOT_USED(box);
+	OPJ_ARG_NOT_USED(jp2);
+
 /* Part 1, I.5.3.6: 'The shall be at most one Channel Definition box
  * inside a JP2 Header box.' 
 */
-	if(color->jp2_cdef) return false;
+	if(color->jp2_cdef) return OPJ_FALSE;
 
-	if((n = cio_read(cio, 2)) == 0) return false; /* szukw000: FIXME */
+	if((n = (unsigned short)cio_read(cio, 2)) == 0) return OPJ_FALSE; /* szukw000: FIXME */
 
 	info = (opj_jp2_cdef_info_t*)
 	 opj_malloc(n * sizeof(opj_jp2_cdef_info_t));
@@ -536,15 +574,15 @@ static bool jp2_read_cdef(opj_jp2_t *jp2, opj_cio_t *cio,
 
 	for(i = 0; i < n; ++i)
    {
-	info[i].cn = cio_read(cio, 2);
-	info[i].typ = cio_read(cio, 2);
-	info[i].asoc = cio_read(cio, 2);
+	info[i].cn = (unsigned short)cio_read(cio, 2);
+	info[i].typ = (unsigned short)cio_read(cio, 2);
+	info[i].asoc = (unsigned short)cio_read(cio, 2);
 
    }
-	return true;
+	return OPJ_TRUE;
 }/* jp2_read_cdef() */
 
-static bool jp2_read_colr(opj_jp2_t *jp2, opj_cio_t *cio,
+static opj_bool jp2_read_colr(opj_jp2_t *jp2, opj_cio_t *cio,
 	opj_jp2_box_t *box, opj_jp2_color_t *color) 
 {
 	int skip_len;
@@ -553,7 +591,7 @@ static bool jp2_read_colr(opj_jp2_t *jp2, opj_cio_t *cio,
 /* Part 1, I.5.3.3 : 'A conforming JP2 reader shall ignore all Colour
  * Specification boxes after the first.' 
 */
-	if(color->jp2_has_colr) return false;
+	if(color->jp2_has_colr) return OPJ_FALSE;
 
 	cinfo = jp2->cinfo;
 
@@ -565,14 +603,15 @@ static bool jp2_read_colr(opj_jp2_t *jp2, opj_cio_t *cio,
    {
 	jp2->enumcs = cio_read(cio, 4);	/* EnumCS */
    } 
-	else 
+	else
+	if (jp2->meth == 2) 
    {
 /* skip PROFILE */
 	skip_len = box->init_pos + box->length - cio_tell(cio);
 	if (skip_len < 0) 
   {
 	opj_event_msg(cinfo, EVT_ERROR, "Error with COLR box size\n");
-	return false;
+	return OPJ_FALSE;
   }
 	if(skip_len > 0)
   {
@@ -591,17 +630,17 @@ static bool jp2_read_colr(opj_jp2_t *jp2, opj_cio_t *cio,
 	if (cio_tell(cio) - box->init_pos != box->length) 
    {
 	opj_event_msg(cinfo, EVT_ERROR, "Error with COLR Box\n");
-	return false;
+	return OPJ_FALSE;
    }
 	color->jp2_has_colr = 1;
 
-	return true;
+	return OPJ_TRUE;
 }/* jp2_read_colr() */
 
-bool jp2_read_jp2h(opj_jp2_t *jp2, opj_cio_t *cio, opj_jp2_color_t *color) 
+opj_bool jp2_read_jp2h(opj_jp2_t *jp2, opj_cio_t *cio, opj_jp2_color_t *color) 
 {
 	opj_jp2_box_t box;
-	unsigned int jp2h_end;
+	int jp2h_end;
 
 	opj_common_ptr cinfo = jp2->cinfo;
 
@@ -613,24 +652,24 @@ bool jp2_read_jp2h(opj_jp2_t *jp2, opj_cio_t *cio, opj_jp2_color_t *color)
 	if (box.type == JP2_JP2C) 
  {
 	opj_event_msg(cinfo, EVT_ERROR, "Expected JP2H Marker\n");
-	return false;
+	return OPJ_FALSE;
  }
 	cio_skip(cio, box.length - 8);
 
-	if(cio->bp >= cio->end) return false;
+	if(cio->bp >= cio->end) return OPJ_FALSE;
 
 	jp2_read_boxhdr(cinfo, cio, &box);
   }
    } while(JP2_JP2H != box.type);
 
 	if (!jp2_read_ihdr(jp2, cio))
-		return false;
+		return OPJ_FALSE;
 	jp2h_end = box.init_pos + box.length;
 
 	if (jp2->bpc == 255) 
    {
 	if (!jp2_read_bpcc(jp2, cio))
-		return false;
+		return OPJ_FALSE;
    }
 	jp2_read_boxhdr(cinfo, cio, &box);
 
@@ -646,7 +685,7 @@ bool jp2_read_jp2h(opj_jp2_t *jp2, opj_cio_t *cio, opj_jp2_color_t *color)
     jp2_read_boxhdr(cinfo, cio, &box);
     continue;
   }
-    if(box.type == JP2_CDEF)
+    if(box.type == JP2_CDEF && !jp2->ignore_pclr_cmap_cdef)
   {
     if( !jp2_read_cdef(jp2, cio, &box, color))
  {
@@ -656,7 +695,7 @@ bool jp2_read_jp2h(opj_jp2_t *jp2, opj_cio_t *cio, opj_jp2_color_t *color)
     jp2_read_boxhdr(cinfo, cio, &box);
     continue;
   }
-    if(box.type == JP2_PCLR)
+    if(box.type == JP2_PCLR && !jp2->ignore_pclr_cmap_cdef)
   {
     if( !jp2_read_pclr(jp2, cio, &box, color))
  {
@@ -666,7 +705,7 @@ bool jp2_read_jp2h(opj_jp2_t *jp2, opj_cio_t *cio, opj_jp2_color_t *color)
     jp2_read_boxhdr(cinfo, cio, &box);
     continue;
   }
-    if(box.type == JP2_CMAP)
+    if(box.type == JP2_CMAP && !jp2->ignore_pclr_cmap_cdef)
   {
     if( !jp2_read_cmap(jp2, cio, &box, color))
  {
@@ -685,12 +724,12 @@ bool jp2_read_jp2h(opj_jp2_t *jp2, opj_cio_t *cio, opj_jp2_color_t *color)
 	cio_seek(cio, jp2h_end);
 
 /* Part 1, I.5.3.3 : 'must contain at least one' */
-	return true;//(color->jp2_has_colr == 1);
+	return OPJ_TRUE;//(color->jp2_has_colr == 1);
 
 }/* jp2_read_jp2h() */
 
-opj_image_t* jp2_decode(opj_jp2_t *jp2, opj_cio_t *cio, 
-	opj_codestream_info_t *cstr_info, bool return_indexed) 
+opj_image_t* opj_jp2_decode(opj_jp2_t *jp2, opj_cio_t *cio, 
+	opj_codestream_info_t *cstr_info, opj_bool return_indexed) 
 {
 	opj_common_ptr cinfo;
 	opj_image_t *image = NULL;
@@ -720,8 +759,10 @@ opj_image_t* jp2_decode(opj_jp2_t *jp2, opj_cio_t *cio,
 	opj_event_msg(cinfo, EVT_ERROR, "Failed to decode J2K image\n");
 	return NULL;
    }
+   
+    if (!jp2->ignore_pclr_cmap_cdef){
 
-/* Set Image Color Space */
+    /* Set Image Color Space */
 	if (jp2->enumcs == 16)
 		image->color_space = CLRSPC_SRGB;
 	else if (jp2->enumcs == 17)
@@ -733,36 +774,35 @@ opj_image_t* jp2_decode(opj_jp2_t *jp2, opj_cio_t *cio,
 	else if (jp2->enumcs == 20)
 		image->color_space = CLRSPC_ERGB;
 	else if (jp2->enumcs == 24)
-		image->color_space = CLRSPC_EYCC;
+		image->color_space = CLRSPC_EYCC; 
 	else
 		image->color_space = CLRSPC_UNKNOWN;
 
 	if(color.jp2_cdef)
-	{
-	    jp2_apply_cdef(image, &color);
-	}
-        image->has_palette = false;
+   {
+	jp2_apply_cdef(image, &color);
+   }
+        image->has_palette = OPJ_FALSE;
 	if(color.jp2_pclr)
-    {
-	    /* Part 1, I.5.3.4: Either both or none : */
-			if (return_indexed || !color.jp2_pclr->cmap)
-             {
-                jp2_free_pclr(&color);
-                image->has_palette = true;
-            }
-            else
-                jp2_apply_pclr(&color, image, cinfo);
-    }
-
+   {
+/* Part 1, I.5.3.4: Either both or none : */
+	if( return_indexed || !color.jp2_pclr->cmap) {
+	 jp2_free_pclr(&color);
+	 image->has_palette = OPJ_TRUE;
+	} else
+	 jp2_apply_pclr(&color, image, cinfo);
+   }
 	if(color.icc_profile_buf)
-        {
-            image->icc_profile_buf = color.icc_profile_buf;
-            color.icc_profile_buf = NULL;
-            image->icc_profile_len = color.icc_profile_len;
-        }
+   {
+	image->icc_profile_buf = color.icc_profile_buf;
+	color.icc_profile_buf = NULL;
+	image->icc_profile_len = color.icc_profile_len;
+   }
+   }
+   
 	return image;
 
-}/* jp2_decode() */
+}/* opj_jp2_decode() */
 
 
 void jp2_write_jp2h(opj_jp2_t *jp2, opj_cio_t *cio) {
@@ -806,7 +846,7 @@ static void jp2_write_ftyp(opj_jp2_t *jp2, opj_cio_t *cio) {
 	cio_seek(cio, box.init_pos + box.length);
 }
 
-static bool jp2_read_ftyp(opj_jp2_t *jp2, opj_cio_t *cio) {
+static opj_bool jp2_read_ftyp(opj_jp2_t *jp2, opj_cio_t *cio) {
 	int i;
 	opj_jp2_box_t box;
 
@@ -816,7 +856,7 @@ static bool jp2_read_ftyp(opj_jp2_t *jp2, opj_cio_t *cio) {
 
 	if (JP2_FTYP != box.type) {
 		opj_event_msg(cinfo, EVT_ERROR, "Expected FTYP Marker\n");
-		return false;
+		return OPJ_FALSE;
 	}
 
 	jp2->brand = cio_read(cio, 4);		/* BR */
@@ -830,10 +870,10 @@ static bool jp2_read_ftyp(opj_jp2_t *jp2, opj_cio_t *cio) {
 
 	if (cio_tell(cio) - box.init_pos != box.length) {
 		opj_event_msg(cinfo, EVT_ERROR, "Error with FTYP Box\n");
-		return false;
+		return OPJ_FALSE;
 	}
 
-	return true;
+	return OPJ_TRUE;
 }
 
 static int jp2_write_jp2c(opj_jp2_t *jp2, opj_cio_t *cio, opj_image_t *image, opj_codestream_info_t *cstr_info) {
@@ -865,7 +905,7 @@ static int jp2_write_jp2c(opj_jp2_t *jp2, opj_cio_t *cio, opj_image_t *image, op
 	return box.length;
 }
 
-static bool jp2_read_jp2c(opj_jp2_t *jp2, opj_cio_t *cio, unsigned int *j2k_codestream_length, unsigned int *j2k_codestream_offset) {
+static opj_bool jp2_read_jp2c(opj_jp2_t *jp2, opj_cio_t *cio, unsigned int *j2k_codestream_length, unsigned int *j2k_codestream_offset) {
 	opj_jp2_box_t box;
 
 	opj_common_ptr cinfo = jp2->cinfo;
@@ -881,7 +921,7 @@ static bool jp2_read_jp2c(opj_jp2_t *jp2, opj_cio_t *cio, unsigned int *j2k_code
 	*j2k_codestream_offset = cio_tell(cio);
 	*j2k_codestream_length = box.length - 8;
 
-	return true;
+	return OPJ_TRUE;
 }
 
 static void jp2_write_jp(opj_cio_t *cio) {
@@ -898,7 +938,7 @@ static void jp2_write_jp(opj_cio_t *cio) {
 	cio_seek(cio, box.init_pos + box.length);
 }
 
-static bool jp2_read_jp(opj_jp2_t *jp2, opj_cio_t *cio) {
+static opj_bool jp2_read_jp(opj_jp2_t *jp2, opj_cio_t *cio) {
 	opj_jp2_box_t box;
 
 	opj_common_ptr cinfo = jp2->cinfo;
@@ -906,35 +946,96 @@ static bool jp2_read_jp(opj_jp2_t *jp2, opj_cio_t *cio) {
 	jp2_read_boxhdr(cinfo, cio, &box);
 	if (JP2_JP != box.type) {
 		opj_event_msg(cinfo, EVT_ERROR, "Expected JP Marker\n");
-		return false;
+		return OPJ_FALSE;
 	}
 	if (0x0d0a870a != cio_read(cio, 4)) {
 		opj_event_msg(cinfo, EVT_ERROR, "Error with JP Marker\n");
-		return false;
+		return OPJ_FALSE;
 	}
 	if (cio_tell(cio) - box.init_pos != box.length) {
 		opj_event_msg(cinfo, EVT_ERROR, "Error with JP Box size\n");
-		return false;
+		return OPJ_FALSE;
 	}
 
-	return true;
+	return OPJ_TRUE;
 }
 
 
-static bool jp2_read_struct(opj_jp2_t *jp2, opj_cio_t *cio,
+static opj_bool jp2_read_struct(opj_jp2_t *jp2, opj_cio_t *cio,
 	opj_jp2_color_t *color) {
 	if (!jp2_read_jp(jp2, cio))
-		return false;
+		return OPJ_FALSE;
 	if (!jp2_read_ftyp(jp2, cio))
-		return false;
+		return OPJ_FALSE;
 	if (!jp2_read_jp2h(jp2, cio, color))
-		return false;
+		return OPJ_FALSE;
 	if (!jp2_read_jp2c(jp2, cio, &jp2->j2k_codestream_length, &jp2->j2k_codestream_offset))
-		return false;
+		return OPJ_FALSE;
 	
-	return true;
+	return OPJ_TRUE;
 }
 
+
+static int write_fidx( int offset_jp2c, int length_jp2c, int offset_idx, int length_idx, opj_cio_t *cio)
+{  
+  int len, lenp;
+  
+  lenp = cio_tell( cio);
+  cio_skip( cio, 4);              /* L [at the end] */
+  cio_write( cio, JPIP_FIDX, 4);  /* IPTR           */
+  
+  write_prxy( offset_jp2c, length_jp2c, offset_idx, length_idx, cio);
+
+  len = cio_tell( cio)-lenp;
+  cio_seek( cio, lenp);
+  cio_write( cio, len, 4);        /* L              */
+  cio_seek( cio, lenp+len);  
+
+  return len;
+}
+
+static void write_prxy( int offset_jp2c, int length_jp2c, int offset_idx, int length_idx, opj_cio_t *cio)
+{
+  int len, lenp;
+
+  lenp = cio_tell( cio);
+  cio_skip( cio, 4);              /* L [at the end] */
+  cio_write( cio, JPIP_PRXY, 4);  /* IPTR           */
+  
+  cio_write( cio, offset_jp2c, 8); /* OOFF           */
+  cio_write( cio, length_jp2c, 4); /* OBH part 1     */
+  cio_write( cio, JP2_JP2C, 4);        /* OBH part 2     */
+  
+  cio_write( cio, 1,1);           /* NI             */
+
+  cio_write( cio, offset_idx, 8);  /* IOFF           */
+  cio_write( cio, length_idx, 4);  /* IBH part 1     */
+  cio_write( cio, JPIP_CIDX, 4);   /* IBH part 2     */
+
+  len = cio_tell( cio)-lenp;
+  cio_seek( cio, lenp);
+  cio_write( cio, len, 4);        /* L              */
+  cio_seek( cio, lenp+len);
+}
+
+static void write_iptr( int offset, int length, opj_cio_t *cio)
+{
+  int len, lenp;
+  
+  lenp = cio_tell( cio);
+  cio_skip( cio, 4);              /* L [at the end] */
+  cio_write( cio, JPIP_IPTR, 4);  /* IPTR           */
+  
+  cio_write( cio, offset, 8);
+  cio_write( cio, length, 8);
+
+  len = cio_tell( cio)-lenp;
+  cio_seek( cio, lenp);
+  cio_write( cio, len, 4);        /* L             */
+  cio_seek( cio, lenp+len);
+}
+
+
 /* ----------------------------------------------------------------------- */
 /* JP2 decoder interface                                             */
 /* ----------------------------------------------------------------------- */
@@ -972,6 +1073,7 @@ void jp2_setup_decoder(opj_jp2_t *jp2, opj_dparameters_t *parameters) {
 	/* setup the J2K codec */
 	j2k_setup_decoder(jp2->j2k, parameters);
 	/* further JP2 initializations go here */
+	jp2->ignore_pclr_cmap_cdef = parameters->flags & OPJ_DPARAMETERS_IGNORE_PCLR_CMAP_CDEF_FLAG;
 }
 
 /* ----------------------------------------------------------------------- */
@@ -1061,30 +1163,23 @@ void jp2_setup_encoder(opj_jp2_t *jp2, opj_cparameters_t *parameters, opj_image_
 	for (i = 0; i < image->numcomps; i++) {
 		jp2->comps[i].bpcc = image->comps[i].prec - 1 + (image->comps[i].sgnd << 7);
 	}
-
-	/* Colour Specification box */
-
-	if ((image->numcomps == 1 || image->numcomps == 3) && (jp2->bpc != 255)) {
-		jp2->meth = 1;	/* METH: Enumerated colourspace */
-	} else {
-		jp2->meth = 2;	/* METH: Restricted ICC profile */
-	}
-	if (jp2->meth == 1) {
-		if (image->color_space == 1)
-			jp2->enumcs = 16;	/* sRGB as defined by IEC 6196621 */
-		else if (image->color_space == 2)
-			jp2->enumcs = 17;	/* greyscale */
-		else if (image->color_space == 3)
-			jp2->enumcs = 18;	/* YUV */
-	} else {
-		jp2->enumcs = 0;		/* PROFILE (??) */
-	}
+	jp2->meth = 1;
+	if (image->color_space == 1)
+		jp2->enumcs = 16;	/* sRGB as defined by IEC 61966-2.1 */
+	else if (image->color_space == 2)
+		jp2->enumcs = 17;	/* greyscale */
+	else if (image->color_space == 3)
+		jp2->enumcs = 18;	/* YUV */
 	jp2->precedence = 0;	/* PRECEDENCE */
 	jp2->approx = 0;		/* APPROX */
-
+	
+	jp2->jpip_on = parameters->jpip_on;
 }
 
-bool jp2_encode(opj_jp2_t *jp2, opj_cio_t *cio, opj_image_t *image, opj_codestream_info_t *cstr_info) {
+opj_bool opj_jp2_encode(opj_jp2_t *jp2, opj_cio_t *cio, opj_image_t *image, opj_codestream_info_t *cstr_info) {
+
+	int pos_iptr, pos_cidx, pos_jp2c, len_jp2c, len_cidx, end_pos, pos_fidx, len_fidx;
+	pos_jp2c = pos_iptr = -1; /* remove a warning */
 
 	/* JP2 encoding */
 
@@ -1095,14 +1190,34 @@ bool jp2_encode(opj_jp2_t *jp2, opj_cio_t *cio, opj_image_t *image, opj_codestre
 	/* JP2 Header box */
 	jp2_write_jp2h(jp2, cio);
 
-	/* J2K encoding */
-
-	if(!jp2_write_jp2c(jp2, cio, image, cstr_info)) {
-		opj_event_msg(jp2->cinfo, EVT_ERROR, "Failed to encode image\n");
-		return false;
+	if( jp2->jpip_on){
+	  pos_iptr = cio_tell( cio);
+	  cio_skip( cio, 24); /* IPTR further ! */
+	  
+	  pos_jp2c = cio_tell( cio);
 	}
 
-	return true;
+	/* J2K encoding */
+	if(!(len_jp2c = jp2_write_jp2c( jp2, cio, image, cstr_info))){
+	    opj_event_msg(jp2->cinfo, EVT_ERROR, "Failed to encode image\n");
+	    return OPJ_FALSE;
+	}
+
+	if( jp2->jpip_on){
+	  pos_cidx = cio_tell( cio);
+	  
+	  len_cidx = write_cidx( pos_jp2c+8, cio, image, *cstr_info, len_jp2c-8);
+	  
+	  pos_fidx = cio_tell( cio);
+	  len_fidx = write_fidx( pos_jp2c, len_jp2c, pos_cidx, len_cidx, cio);
+	  
+	  end_pos = cio_tell( cio);
+	  
+	  cio_seek( cio, pos_iptr);
+	  write_iptr( pos_fidx, len_fidx, cio);
+	  
+	  cio_seek( cio, end_pos);
+	}
+
+	return OPJ_TRUE;
 }
-
-
