@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2007 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Command line parsing and dispatching */
 #include "ctype_.h"
 #include "memory_.h"
@@ -132,7 +134,9 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
     int code;
 
     arg_init(&args, (const char **)argv, argc,
-             gs_main_arg_fopen, (void *)minst);
+             gs_main_arg_fopen, (void *)minst,
+             minst->get_codepoint,
+             minst->heap);
     code = gs_main_init0(minst, 0, 0, 0, GS_MAX_LIB_DIRS);
     if (code < 0)
         return code;
@@ -165,15 +169,18 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
         bool helping = false;
 
         for (i = 1; i < argc; ++i)
-            if (!strcmp(argv[i], "--")) {
+            if (!arg_strcmp(&args, argv[i], "--")) {
                 /* A PostScript program will be interpreting all the */
                 /* remaining switches, so stop scanning. */
                 helping = false;
                 break;
-            } else if (!strcmp(argv[i], "--help")) {
+            } else if (!arg_strcmp(&args, argv[i], "--help")) {
                 print_help(minst);
                 helping = true;
-            } else if (!strcmp(argv[i], "--version")) {
+            } else if (!arg_strcmp(&args, argv[i], "--debug")) {
+                gs_debug_flags_list(minst->heap);
+                helping = true;
+            } else if (!arg_strcmp(&args, argv[i], "--version")) {
                 print_version(minst);
                 puts(minst->heap, "");  /* \n */
                 helping = true;
@@ -196,11 +203,16 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
             (char *)gs_alloc_bytes(minst->heap, len, "GS_OPTIONS");
 
             gp_getenv(GS_OPTIONS, opts, &len);  /* can't fail */
+#if defined(GS_NO_UTF8)
             if (arg_push_memory_string(&args, opts, false, minst->heap))
                 return e_Fatal;
+#else
+            if (arg_push_decoded_memory_string(&args, opts, false, true, minst->heap))
+                return e_Fatal;
+#endif
         }
     }
-    while ((arg = arg_next(&args, &code)) != 0) {
+    while ((arg = arg_next(&args, &code, minst->heap)) != 0) {
         switch (*arg) {
             case '-':
                 code = swproc(minst, arg, &args);
@@ -211,10 +223,10 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
                 if (gs_debug[':'] && arg[1] == 'Z') {
                     int i;
 
-                    dprintf1("%% Init started, instance 0x%p, with args: ", minst);
+                    dmprintf1(minst->heap, "%% Init started, instance 0x%p, with args: ", minst);
                     for (i=1; i<argc; i++)
-                        dprintf1("%s ", argv[i]);
-                    dprintf("\n");
+                        dmprintf1(minst->heap, "%s ", argv[i]);
+                    dmprintf(minst->heap, "\n");
                 }
                 break;
             default:
@@ -233,16 +245,30 @@ gs_main_init_with_args(gs_main_instance * minst, int argc, char *argv[])
     if (gs_debug[':']) {
         int i;
 
-        dprintf1("%% Init done, instance 0x%p, with args: ", minst);
+        dmprintf1(minst->heap, "%% Init done, instance 0x%p, with args: ", minst);
         for (i=1; i<argc; i++)
-            dprintf1("%s ", argv[i]);
-        dprintf("\n");
+            dmprintf1(minst->heap, "%s ", argv[i]);
+        dmprintf(minst->heap, "\n");
     }
     if (!minst->run_start)
         return e_Quit;
     return code ;
 }
 
+/*
+ * Specify a decoding function
+ */
+void gs_main_inst_arg_decode(gs_main_instance * minst,
+                             gs_arg_get_codepoint *get_codepoint)
+{
+    minst->get_codepoint = get_codepoint;
+}
+
+gs_arg_get_codepoint *gs_main_inst_get_arg_decode(gs_main_instance * minst)
+{
+    return minst->get_codepoint;
+}
+
 /*
  * Run the 'start' procedure (after processing the command line).
  */
@@ -290,11 +316,19 @@ run_stdin:
                 return code;
             break;
         case '-':               /* run with command line args */
+            if (strncmp(arg, "debug=", 6) == 0) {
+                code = gs_debug_flags_parse(minst->heap, arg+6);
+                if (code < 0)
+                    return code;
+                break;
+            }
+            /* FALLTHROUGH */
         case '+':
             pal->expand_ats = false;
+            /* FALLTHROUGH */
         case '@':               /* ditto with @-expansion */
             {
-                const char *psarg = arg_next(pal, &code);
+                const char *psarg = arg_next(pal, &code, minst->heap);
 
                 if (code < 0)
                     return e_Fatal;
@@ -305,27 +339,24 @@ run_stdin:
                 }
                 psarg = arg_copy(psarg, minst->heap);
                 if (psarg == NULL)
-                    return e_Fatal;
-                code = gs_main_init2(minst);
-                if (code < 0)
-                    return code;
-                code = run_string(minst, "userdict/ARGUMENTS[", 0);
-                if (code < 0)
-                    return code;
-                while ((arg = arg_next(pal, &code)) != 0) {
-                    char *fname = arg_copy(arg, minst->heap);
-                    if (fname == NULL)
-                        return e_Fatal;
-                    code = runarg(minst, "", fname, "", runInit);
-                    if (code < 0)
-                        return code;
-                }
-                if (code < 0)
-                    return e_Fatal;
-                code = runarg(minst, "]put", psarg, ".runfile", runInit | runFlush);
-                if (code < 0)
-                    return code;
-                return e_Quit;
+                    code = e_Fatal;
+                else
+                    code = gs_main_init2(minst);
+                if (code >= 0)
+                    code = run_string(minst, "userdict/ARGUMENTS[", 0);
+                if (code >= 0)
+                    while ((arg = arg_next(pal, &code, minst->heap)) != 0) {
+                        code = runarg(minst, "", arg, "", runInit);
+                        if (code < 0)
+                            break;
+                    }
+                if (code >= 0)
+                    code = runarg(minst, "]put", psarg, ".runfile", runInit | runFlush);
+                arg_free((char *)psarg, minst->heap);
+                if (code >= 0)
+                    code = e_Quit;
+                
+                return code;
             }
         case 'A':               /* trace allocator */
             switch (*arg) {
@@ -365,17 +396,12 @@ run_stdin:
                 if (code < 0)
                     return code;
                 pal->expand_ats = false;
-                while ((arg = arg_next(pal, &code)) != 0) {
-                    char *sarg;
-
+                while ((arg = arg_next(pal, &code, minst->heap)) != 0) {
                     if (arg[0] == '@' ||
                         (arg[0] == '-' && !isdigit((unsigned char)arg[1]))
                         )
                         break;
-                    sarg = arg_copy(arg, minst->heap);
-                    if (sarg == NULL)
-                        return e_Fatal;
-                    code = runarg(minst, "", sarg, ".runstring", 0);
+                    code = runarg(minst, "", arg, ".runstring", runBuffer);
                     if (code < 0)
                         return code;
                 }
@@ -448,7 +474,17 @@ run_stdin:
             return e_Info;      /* show usage info on exit */
         case 'I':               /* specify search path */
             {
-                char *path = arg_copy(arg, minst->heap);
+                const char *path;
+
+                if (arg[0] == 0) {
+                    path = arg_next(pal, &code, minst->heap);
+                    if (code < 0)
+                        return code;
+                } else
+                    path = arg;
+                if (path == NULL)
+                    return e_Fatal;
+                path = arg_copy(path, minst->heap);
                 if (path == NULL)
                     return e_Fatal;
                 gs_main_add_lib_path(minst, path);
@@ -504,7 +540,7 @@ run_stdin:
                 int len;
 
                 if (arg[0] == 0) {
-                    adef = arg_next(pal, &code);
+                    adef = arg_next(pal, &code, minst->heap);
                     if (code < 0)
                         return code;
                 } else
@@ -513,6 +549,8 @@ run_stdin:
                     return code;
                 len = strlen(adef);
                 str = (char *)gs_alloc_bytes(minst->heap, (uint)len, "-o");
+                if (str == NULL)
+                    return e_VMerror;
                 memcpy(str, adef, len);
                 make_const_string(&value, a_readonly | avm_foreign,
                                   len, (const byte *)str);
@@ -579,7 +617,7 @@ run_stdin:
                 if ((code = gs_main_init1(minst)) < 0)
                     return code;
                 if (eqp == adef) {
-                    puts(minst->heap, "Usage: -dname, -dname=token, -sname=string");
+                    puts(minst->heap, "Usage: -dNAME, -dNAME=TOKEN, -sNAME=STRING");
                     return e_Fatal;
                 }
                 if (eqp == NULL) {
@@ -595,38 +633,69 @@ run_stdin:
                     *eqp++ = 0;
                     ialloc_set_space(idmemory, avm_system);
                     if (isd) {
-                        stream astream;
-                        scanner_state state;
+                        int num, i;
 
-                        s_init(&astream, NULL);
-                        sread_string(&astream,
-                                     (const byte *)eqp, strlen(eqp));
-                        gs_scanner_init_stream(&state, &astream);
-                        code = gs_scan_token(minst->i_ctx_p, &value, &state);
-                        if (code) {
-                            puts(minst->heap, "-dname= must be followed by a valid token");
-                            return e_Fatal;
-                        }
-                        if (r_has_type_attrs(&value, t_name,
-                                             a_executable)) {
-                            ref nsref;
+                        /* Check for numbers so we can provide for suffix scalers */
+                        /* Note the check for '#' is for PS "radix" numbers such as 16#ff */
+                        /* and check for '.' and 'e' or 'E' which are 'real' numbers */
+                        if ((strchr(eqp, '#') == NULL) && (strchr(eqp, '.') == NULL) &&
+                            (strchr(eqp, 'e') == NULL) && (strchr(eqp, 'E') == NULL) &&
+                            ((i = sscanf((const char *)eqp, "%d", &num)) == 1)) {
+                            char suffix = eqp[strlen(eqp) - 1];
 
-                            name_string_ref(minst->heap, &value, &nsref);
-#define string_is(nsref, str, len)\
-  (r_size(&(nsref)) == (len) &&\
-   !strncmp((const char *)(nsref).value.const_bytes, str, (len)))
-                            if (string_is(nsref, "null", 4))
-                                make_null(&value);
-                            else if (string_is(nsref, "true", 4))
-                                make_true(&value);
-                            else if (string_is(nsref, "false", 5))
-                                make_false(&value);
-                            else {
-                                puts(minst->heap,
-                                     "-dvar=name requires name=null, true, or false");
+                            switch (suffix) {
+                                case 'k':
+                                case 'K':
+                                    num *= 1024;
+                                    break;
+                                case 'm':
+                                case 'M':
+                                    num *= 1024 * 1024;
+                                    break;
+                                case 'g':
+                                case 'G':
+                                    /* caveat emptor: more than 2g will overflow */
+                                    /* and really should produce a 'real', so don't do this */
+                                    num *= 1024 * 1024 * 1024;
+                                    break;
+                                default:
+                                    break;   /* not a valid suffix or last char was digit */
+                            }
+                            make_int(&value, num);
+                        } else {
+                            /* use the PS scanner to capture other valid token types */
+                            stream astream;
+                            scanner_state state;
+
+                            s_init(&astream, NULL);
+                            sread_string(&astream,
+                                         (const byte *)eqp, strlen(eqp));
+                            gs_scanner_init_stream(&state, &astream);
+                            code = gs_scan_token(minst->i_ctx_p, &value, &state);
+                            if (code) {
+                                outprintf(minst->heap, "Invalid value for option -d%s, -dNAME= must be followed by a valid token\n", arg);
                                 return e_Fatal;
                             }
-#undef name_is_string
+                            if (r_has_type_attrs(&value, t_name,
+                                                 a_executable)) {
+                                ref nsref;
+
+                                name_string_ref(minst->heap, &value, &nsref);
+#undef string_is
+#define string_is(nsref, str, len)\
+       (r_size(&(nsref)) == (len) &&\
+       !strncmp((const char *)(nsref).value.const_bytes, str, (len)))
+                                if (string_is(nsref, "null", 4))
+                                    make_null(&value);
+                                else if (string_is(nsref, "true", 4))
+                                    make_true(&value);
+                                else if (string_is(nsref, "false", 5))
+                                    make_false(&value);
+                                else {
+                                    outprintf(minst->heap, "Invalid value for option -d%s, use -sNAME= to define string constants\n", arg);
+                                    return e_Fatal;
+                                }
+                            }
                         }
                     } else {
                         int len = strlen(eqp);
@@ -731,19 +800,15 @@ static int
 argproc(gs_main_instance * minst, const char *arg)
 {
     int code = gs_main_init1(minst);            /* need i_ctx_p to proceed */
-    char *filearg;
 
     if (code < 0)
         return code;
-    filearg = arg_copy(arg, minst->heap);
-    if (filearg == NULL)
-        return e_Fatal;
     if (minst->run_buffer_size) {
         /* Run file with run_string. */
-        return run_buffered(minst, filearg);
+        return run_buffered(minst, arg);
     } else {
         /* Run file directly in the normal way. */
-        return runarg(minst, "", filearg, ".runfile", runInit | runFlush);
+        return runarg(minst, "", arg, ".runfile", runInit | runFlush);
     }
 }
 static int
@@ -801,7 +866,7 @@ runarg(gs_main_instance * minst, const char *pre, const char *arg,
         if (code < 0)
             return code;
     }
-    line = (char *)gs_alloc_bytes(minst->heap, len, "argproc");
+    line = (char *)gs_alloc_bytes(minst->heap, len, "runarg");
     if (line == 0) {
         lprintf("Out of memory!\n");
         return_error(e_VMerror);
@@ -809,9 +874,19 @@ runarg(gs_main_instance * minst, const char *pre, const char *arg,
     strcpy(line, pre);
     esc_strcat(line, arg);
     strcat(line, post);
-    minst->i_ctx_p->starting_arg_file = true;
+    /* If we're running PS from a buffer (i.e. from the "-c" option
+       we don't want lib_file_open() to search the current directory
+       during this call to run_string()
+     */
+    if ((options & runBuffer)) {
+        minst->i_ctx_p->starting_arg_file = false;
+    }
+    else {
+        minst->i_ctx_p->starting_arg_file = true;
+    }
     code = run_string(minst, line, options);
     minst->i_ctx_p->starting_arg_file = false;
+    gs_free_object(minst->heap, line, "runarg");
     return code;
 }
 static int
@@ -904,6 +979,11 @@ static const char help_usage2[] = "\
  -sDEVICE=<devname>  select device         | -dBATCH  exit after last file\n\
  -sOutputFile=<file> select output file: - for stdout, |command for pipe,\n\
                                          embed %d or %ld for page #\n";
+#ifdef DEBUG
+static const char help_debug[] = "\
+ --debug=<option>[,<option>]*  select debugging options\n\
+ --debug                       list debugging options\n";
+#endif
 static const char help_trailer[] = "\
 For more information, see %s.\n\
 Please report bugs to bugs.ghostscript.com.\n";
@@ -911,6 +991,11 @@ static const char help_devices[] = "Available devices:";
 static const char help_default_device[] = "Default output device:";
 static const char help_emulators[] = "Input formats:";
 static const char help_paths[] = "Search path:";
+#ifdef HAVE_FONTCONFIG
+static const char help_fontconfig[] = "Ghostscript is also using fontconfig to search for font files\n";
+#else
+static const char help_fontconfig[] = "";
+#endif
 
 extern_gx_io_device_table();
 
@@ -966,6 +1051,9 @@ print_usage(const gs_main_instance *minst)
 {
     outprintf(minst->heap, "%s", help_usage1);
     outprintf(minst->heap, "%s", help_usage2);
+#ifdef DEBUG
+    outprintf(minst->heap, "%s", help_debug);
+#endif
 }
 
 /* compare function for qsort */
@@ -1028,17 +1116,10 @@ print_emulators(const gs_main_instance *minst)
 {
     outprintf(minst->heap, "%s", help_emulators);
     {
-        const ref *pes;
+        const byte *s;
 
-        for (pes = gs_emulator_name_array;
-             pes->value.const_bytes != 0; pes++
-            )
-            /*
-             * Even though gs_emulator_name_array is declared and used as
-             * an array of string refs, each string is actually a
-             * (null terminated) C string.
-             */
-            outprintf(minst->heap, " %s", (const char *)pes->value.const_bytes);
+        for (s = gs_emulators; s[0] != 0; s += strlen((const char *)s) + 1)
+            outprintf(minst->heap, " %s", s);
     }
     outprintf(minst->heap, "\n");
 }
@@ -1084,6 +1165,7 @@ print_paths(gs_main_instance * minst)
         }
     }
     outprintf(minst->heap, "\n");
+    outprintf(minst->heap, "%s", help_fontconfig);
 }
 
 /* Print the help trailer. */
